//#include "metalShaderTypes.h"

typedef struct
{
	float4 darkColor;
	float4 brightColor;
} fxVars;

vertex fxVertexOut vertexFunc(uint vertexID [[ vertex_id ]]
							  ,const device fxShaderVerts* in[[ buffer(0) ]]
							  ,texture2d<half> inputTex0 [[ texture(0) ]]
							  ,texture2d<half> inputTexMask [[ texture(1) ]]
							  ,constant fxGeneralUniforms& u[[ buffer(1) ]]
							  ,constant fxVars& vars[[ buffer(2)]])
{
    fxVertexOut out;

	out.normPos=in[vertexID].pos.xy*vec2(.5,-.5f);//+vec2(.5,.5);
	out.position=in[vertexID].pos;
	out.texSize=float2(inputTex0.get_width(),inputTex0.get_height());
	out.tex1=in[vertexID].tex1;
	out.texMask=in[vertexID].texMask;

    return out;
}

fragment float4 fragmentFunc(fxVertexOut in [[stage_in]]
							 ,texture2d<half> inputTex [[ texture(0) ]]
							 ,texture2d<half> inputTexMask [[ texture(1) ]]
							 ,constant fxGeneralUniforms& u[[ buffer(0) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
	vec4 texel = tex(inputTex, fsTexture);
	// Convert to grayscale using NTSC conversion weights
	float gray = dot(texel.rgb, vec3(0.299, 0.587, 0.114));
	
	// linear interp between darkColor and brightColor using gray scale
	float4 outColor;
	outColor.rgb=mix(vars.darkColor.rgb,vars.brightColor.rgb,gray);
    outColor.a=texel.a;
	
	float blendValue=u.blendValue;
#ifdef USE_MASK
	blendValue*=tex(in.texMask,inputTexMask).a;
#endif
	outColor.rgb=mix(texel.rgb,outColor.rgb,blendValue);
	return outColor;
}
